;*********************************************************************
; COMMODORE VIC 20 BASIC BOOT AND STARTUP
; for machines with at least 8K memory expansion
;
		.fileopt author,	"Robert Hurst"
        .fileopt comment,	"Quikman+"
        .fileopt compiler,	"VIC 20 ASSEMBLER"

;*********************************************************************
; VIC 20 boot (8K or more memory expansion) from BASIC system startup
; written by Robert Hurst <spock@hurst-ri.us>
; updated version: 11-Oct-2009
;
		.include "vic-sss4.h"
		.include "mazedata.h"
		.global GENDER
		.global LIVES
		.global MENU
		.global MEGACART
		.global NVRAM
		.global PLAYERS

;*********************************************************************
; Commodore BASIC loader
;
; LOAD "quikman+8k.prg",8
;
		.segment "BOOT"
		.word	$1201	; added to VIC binary: fixed-address loader
BASIC:	.byte	$0A, $12
				; 2009 SYS4621
		.byte	$D9, $07, $9E, $34, $36, $32, $31
		.byte	$00, $00, $00
		;
MAIN:
		LDA MACHINE
		CMP #$05
		BEQ NTSC
		CMP #$0C
		BEQ PAL
		JMP RESET		; not a VIC?
		;
		; NTSC setup
NTSC:	LDX #<@NTSC		; load the timer low-byte latches
		STX $9126
		LDX #>@NTSC
		LDA #$75		; raster line 234/235
		BNE IRQSYNC
@NTSC = $4243			; (261 * 65 - 2)
		;
		; PAL setup
PAL:	LDX #<@PAL		; load the timer low-byte latches
		STX $9126
		LDX #>@PAL
		LDA #$82		; raster line 260/261
		BNE IRQSYNC
@PAL = $5686			; (312 * 71 - 2)
		;
IRQSYNC:
		CMP VIC+$04
		BNE IRQSYNC
		STX $9125		; load T1 latch high, and transfer both bytes to T1 counter
		; init VIC
		INC VIC			; adjust left border to accommodate one less column
		LDA VIC+$01
		SEC
		SBC #$04		; adjust top scan line to accomodate extra row
		STA VIC+$01
		LDA #$00+$15	; set for videoram @ $1400 with 21-columns
		STA VIC+$02		; video matrix address + columns
		LDA #$B0		; $B0 = 10110000 = 24 rows + 8x8 height
		STA VIC+$03		; rows / character height
		LDA #$DF		; set video @ $1400 and char table @ $1C00
		STA VIC+$05
		; reset sound channels
@cont:	LDA #$00
		TAY
@snd:	STA VIC+$0A,Y
		INY
		CPY #$04
		BNE @snd
		LDA #$8F		; brown & highest
		STA VIC+$0E		; auxiliary color & volume
		LDA #$80
		STA SHIFTMODE	; locked
		;
@hi:	LDA #$FE
		STA MEGACART	; init MC memory register
		STA VIC+$0F		; lt yellow screen / blue border
		STA MEGACART	; init MC memory register a 2nd time
		CMP MEGACART	; detect if this memory location was really writable
		BNE SPLASH		; no, might be an emulator or a real VIC with only 8k
		LDX NVRAM+2		; load saved high score
		LDY NVRAM+1
		TXA
		AND #$0F
		BNE MC			; no one's allowed -- reset hi-score
		LDA NVRAM
		CMP #$02
		BCC MC			; less than 20,000 points?
		STA HISCORE
		STY HISCORE+1
		STX HISCORE+2
		JMP MC
		;
SPLASH:	DEC VIC+$0F		; lt yellow screen / green border
MC:		; my interrupt vector init
		SEI
		LDX #<BACKGROUND
		LDY #>BACKGROUND
		STX $0314
		STY $0315
		CLI

		LDX #<SPLASHCOLOR
		LDY #>SPLASHCOLOR
		STX VECTORFG
		STY VECTORFG+1
		LDX #$00
		LDY #$94
		STX VECTORBG
		STY VECTORBG+1
		LDX #$02
		LDY #$00
@fill:	LDA (VECTORFG),Y
		STA (VECTORBG),Y
		INY
		BNE @fill
		INC VECTORFG+1
		INC VECTORBG+1
		DEX
		BNE @fill
		LDY #$94
		STY VECTORBG+1
@loop:	LDA JIFFYL
@wait:	CMP JIFFYL
		BEQ @wait
		TAX
		AND #$03
		BNE @loop		; update every 4th jiffy
		TXA
		AND #$0F		; use remainder as x4 (2x2 character matrix)
		CLC
		ADC #$50		; add big quikman base character code
		TAX
		STX $1457
		INX
		STX $146C
		INX
		STX $1458
		INX
		STX $146D
		;
		LDX MARQUEE
		LDA BULBS,X
		TAY
		LDA #$02		; RED
		STA (VECTORBG),Y
		INX
		CPX #$30
		BNE @l0
		LDX #$00
@l0:	STX MARQUEE
		LDA BULBS,X
		TAY
		LDA #$04		; MAGENTA
		STA (VECTORBG),Y
		;
		LDX MARQUEE+1
		LDA BULBS,X
		TAY
		LDA #$06		; BLUE
		STA (VECTORBG),Y
		INX
		CPX #$30
		BNE @l1
		LDX #$00
@l1:	STX MARQUEE+1
		LDA BULBS,X
		TAY
		LDA #$03		; CYAN
		STA (VECTORBG),Y
		;
		LDX MARQUEE+2
		LDA BULBS,X
		TAY
		LDA #$05		; GREEN
		STA (VECTORBG),Y
		INX
		CPX #$30
		BNE @l2
		LDX #$00
@l2:	STX MARQUEE+2
		LDA BULBS,X
		TAY
		LDA #$07		; YELLOW
		STA (VECTORBG),Y
		;
@key:	JSR GETIN
		CMP #$03		; got STOP ?
		BNE @no
		JMP READY
@no:	CMP #$85		; got F1 ?
		BEQ @go
		LDA #$FF
		STA $9122
		LDA $9111
		AND #$20		; got FIRE ?
		BEQ @go
		jmp @loop
@go:	LDA #$00
		STA GENDER
		STA PLAYERS
		JMP MENU
		;
MARQUEE:
		.byte	0,16,32
BULBS:	.byte	1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19
		.byte	41,62,83,104,125
		.byte	145,144,143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128,127
		.byte	105,84,63,42,21

		.segment "SPLASH"
SPLASHDATA:
		.byte	$1F,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1F
		.byte	$1E,$37,$3A,$38,$31,$20,$31,$31,$31,$31,$20,$20,$20,$20,$20,$20,$4C,$4E,$20,$20,$1E
		.byte	$1E,$39,$20,$39,$39,$20,$39,$39,$3B,$3E,$38,$20,$20,$20,$20,$20,$4D,$4F,$20,$20,$1E
		.byte	$1E,$35,$3D,$36,$35,$3A,$36,$32,$32,$20,$32,$37,$3D,$38,$37,$3A,$38,$37,$3A,$38,$1E
		.byte	$1E,$20,$32,$50,$52,$20,$20,$20,$20,$48,$4A,$39,$32,$39,$3B,$3A,$3C,$39,$20,$39,$1E
		.byte	$1E,$20,$20,$51,$53,$20,$20,$20,$20,$49,$4B,$32,$20,$32,$32,$20,$32,$32,$20,$32,$1E
		.byte	$1F,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1F
		.byte	$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$20,$86,$85,$81,$94,$95,$92,$89,$8E,$87,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$0D,$20,$81,$82,$92,$81,$88,$81,$8D,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$0D,$20,$92,$85,$82,$85,$83,$83,$81,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$0D,$20,$89,$93,$81,$82,$85,$8C,$8C,$81,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$0D,$20,$90,$85,$81,$8E,$95,$94,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$20,$20,$20,$21,$20,$22,$20,$23,$20,$24,$20,$25,$20,$26,$20,$27,$20,$28,$20,$20,$20
		.byte	$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$A0,$A0,$60,$B2,$B0,$B0,$B9,$A0,$92,$8F,$82,$85,$92,$94,$A0,$88,$95,$92,$93,$94,$A0
		.byte	$20,$20,$20,$20,$20,$20,$20,$EC,$E2,$E2,$FB,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.byte	$A0,$A0,$90,$92,$85,$93,$93,$40,$41,$43,$E1,$86,$8F,$92,$A0,$8D,$85,$8E,$95,$A0,$A0
		.byte	$20,$20,$20,$20,$20,$20,$20,$FC,$42,$42,$FE,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
		.res	8
SPLASHCOLOR:
		.byte	$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$09,$09,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$09,$09,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$07,$07,$00,$00,$00,$00,$0A,$0A,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$07,$07,$00,$00,$00,$00,$0A,$0A,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$06,$06,$06,$06,$06,$06,$06,$06,$06,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$02,$00,$02,$02,$02,$02,$02,$02,$02,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$05,$00,$05,$05,$05,$05,$05,$05,$05,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$03,$00,$03,$03,$03,$03,$03,$03,$03,$03,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$07,$00,$07,$07,$07,$07,$07,$07,$07,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$02,$00,$02,$00,$07,$00,$02,$00,$05,$00,$04,$00,$07,$00,$03,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06
		.byte	$00,$00,$00,$00,$00,$00,$00,$09,$09,$09,$09,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$09,$09,$09,$09,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.byte	$00,$00,$00,$00,$00,$00,$00,$09,$09,$09,$09,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		.res	8

;*********************************************************************
; VIC Software Sprite Stack (SSS)
;
; The above BASIC loader will be overwritten by SSS upon its
; initialization (SSSINIT).  The linker will fill this reserved space
; for the dual video frame buffers, play field, and the sprite image buffers:
; 4096 - 6406 ($1000 - $1906)
;
; $1000 - $11FF		VICFRAME1 - first video buffer
; $1200 - $13FF		VICFRAME2 - second video buffer
; $1400 - $15FF		PLAYFIELD - write-pending screen buffer
; $1600 - $17FF		PLAYCOLOR - write-pending color / dirty buffer
;
		.segment "SSSBUF"
;
; this can be resized smaller as required -- if all 64-chars are used by
; sprites, that exhausts all 128 custom characters ($1C00-$1FFF) for
; double-buffering (x2)
SSSBUF:	.res 20 * 8		; this games requires up to 20*2 chars ($58-$7F)
;
; SPRITE REGISTERS
;
SPRITEBUFH:	.res SPRITEMAX	; pointer within sprite image buffer @ above address
SPRITEBUFL:	.res SPRITEMAX	; pointer within sprite image buffer @ above address
SPRITEC1H:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITEC1L:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITEC2H:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITEC2L:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITECOL:	.res SPRITEMAX	; 4-bit VIC color code
SPRITEDEF:	.res SPRITEMAX	; matrix/color definition (see explanation below)
SPRITEH:	.res SPRITEMAX	; number of raster lines (1-16)
SPRITEIMGH:	.res SPRITEMAX	; pointer to source graphic for rendering at 0,0
SPRITEIMGL:	.res SPRITEMAX	; pointer to source graphic for rendering at 0,0
SPRITEX:	.res SPRITEMAX	; horizontal pixel coordinate, visible >0 - <SSSCLIPX
SPRITEY:	.res SPRITEMAX	; vertical pixel coordinate, visible >0 - <SSSCLIPY
SPRITEZ:	.res SPRITEMAX	; bit 0: last rendered (0 = SPRITEC1; 1 = SPRITEC2)
							; bit 5: background is all SSSNULLs (1 = true)
							; bit 6: copy/merge into alternate sprite character pool
							; bit 7: copy/shift sprite image into its buffer
;
; SPRITEDEF is a bit-structure of these characteristics:
; - height		bit 0: 0 = 8px; 1 = 16px
; - width		bit 1: 0 = 8px; 1 = 16px
; - float Y		bit 2: flag: 0=fixed cell, 1=vertical float
; - float X		bit 3: flag: 0=fixed cell, 1=horizontal float
; - double Y	bit 4: double-pixel height (future?)
; - double X	bit 5: double-pixel width (future?)
; - copy		bit 6: copy/merge into alternate sprite character pool
; - render		bit 7: copy/shift sprite image into its buffer
;
SSS:		.res 24*2	; screen row index -- computed by PLAYCOLS
SPRITE:		.byte $00	; number of active sprite registers (0 - SPRITEMAX)
sssNUM:		.byte $00	; current sprite # (0-1)
sssX:		.byte $00	; current sprite width: 0=8w, 1=16w, 2=24w
sssY:		.byte $00	; current sprite height: 0=8h, 1=16h, 2=24h
sssBYTES:	.byte $00	; number of bytes this sprite occupies
sssNEXT:	.byte $00	; offset to adjacent character
sssCHAR:	.byte $00	; next custom character to use on PENDING frame
sssDX:		.byte $00	; delta X counter
sssDY:		.byte $00	; delta Y counter
sssLINE:	.byte $00	; current sprite make line: 0, 8, 16
sssLINENUM:	.byte $00	; current sprite line countdown
sssROR1:	.byte $00	; bit shift register column #1
sssROR2:	.byte $00	; bit shift register column #2
sssROR3:	.byte $00	; bit shift register column #3
sssXFER:	.byte $00	; transfer to custom character counter
sssALLOC:	; 8x8, 16x8, 8x16, 16x16
			.byte	8,16,16,32	; fixed:	1,2,2,4
			.byte	16,24,32,48	; float Y:	2,3,4,6
			.byte	16,32,24,48	; float X:	2,4,3,6
			.byte	32,48,48,72	; both:		4,6,6,9
sssROWS:	.byte	1,2,1,2	; fixed
			.byte	2,3,2,3	; float Y
			.byte	1,2,1,2	; float X
			.byte	2,3,2,3	; both
sssCOLS:	.byte	1,1,2,2	; fixed
			.byte	1,1,2,2	; float Y
			.byte	2,2,3,3	; float X
			.byte	2,2,3,3	; both

;*********************************************************************
; other initialized data can be appended here:
;
			.segment "MYDATA"
INTRO:	.byte	59, 221, 7	; b1 & B4
		.byte	    238, 7	; B5
		.byte	93, 232, 7	; b2 & F#5
		.byte	    228, 7	; D#5
		.byte	59, 238, 7	; b1 & B5
		.byte	    232, 7	; F#5
		.byte	93, 228, 9	; b2 & D#5
		;
		.byte	63, 223, 7	; c2 & C5
		.byte	    239, 7	; C6
		.byte	95, 234, 7	; c3 & G5
		.byte	    230, 7	; E5
		.byte	63, 239, 7	; c2 & C6
		.byte	    234, 7	; G5
		.byte	95, 230, 9	; c3 & E5
		;
		.byte	59, 221, 7	; b1 & B4
		.byte	    238, 7	; B5
		.byte	93, 232, 7	; b2 & F#5
		.byte	    228, 7	; D#5
		.byte	59, 238, 7	; b1 & B5
		.byte	    232, 7	; F#5
		.byte	93, 228, 9	; b2 & D#5
		;
		.byte	82, 228, 6	; f#2 & D#5
		.byte	    230, 6	; E5
		.byte	    231, 3	; F5
		.byte	 0,   0, 1	; mute
		.byte	87, 231, 3	; g#2 & F5
		.byte	    232, 6	; F#5
		.byte	    234, 3	; G5
		.byte	 0,   0, 1	; mute
		.byte	91, 234, 3	; a#2 & G5
		.byte	    235, 6	; G#5
		.byte	    236, 3	; A5
		.byte	 0,   0, 1	; mute
		.byte	93, 236, 3	; b2 & A5
		.byte	    237, 6	; A#5
		.byte	    238, 6	; B5
SNDBIT:	.byte	  0,  0, 0	; mute & fini
		; yummy sound effect
		.byte	$C0, $B8, $B0, $A8, $B0, $B8, $C0, $C8
;
SCORE1:		.byte	0,0,0	; player1
SCORE2:		.byte	0,0,0	; player2
HISCORE:	.byte	2,0,0	; score to beat
LEVEL:		.byte	0,0		; player1&2
STARTING:	.byte	0		; starting level
LIVES:		.byte	0,0		; player1&2
DEMO:		.byte	1		; demo mode
DOTS:		.byte	0,0		; player1&2 dots remaining
FRUITSCORE:
		.byte	$01, $03, $05, $07, $10, $20, $30, $50
CAGEDATA:		; knowledge cycle time
		.byte	$00, $33, $76, $F9
				; right, up, left, right
		.byte	$00, $03, $02, $00
				; coordinate to consider going to upon release or demo
		.byte	$A8, $30, $18, $30, $A8, $98, $18, $98
INERTIA:		; maintain direction
		.byte	$01, $00, $00, $01, $FF, $00, $00, $FF
PEEKAHEAD:		;
		.byte	$01, $15
PEEKAHEAD2:		;
		.byte	$14, $00
STARTPOSX:		;
		.byte	$60, $60, $60, $70, $50
STARTPOSY:		;
		.byte	$98, $58, $68, $68, $68
;
QUIKMANCLR:		; yellow
		.byte	$07
MONSTERCLR:		; red, green, cyan, yellow
		.byte	$02, $05, $03, $07
FRUIT:			; cherry, strawberry, 2-peach, 2-apple, 2-pineapple, 2-tbird, 2-bell, key
		.byte	$21, $22, $23, $23, $24, $24, $25, $25, $26, $26, $27, $27, $28
FRUITCLR:		; red, red, 2-yellow, 2-red, 2-green, 2-magenta, 2-yellow, cyan
		.byte	$02, $02, $07, $07, $02, $02, $05, $05, $04, $04, $07, $07, $03
FRUITCHAR:		;
		.byte	$02, $01, $00	;  100
		.byte	$03, $01, $00	;  300
		.byte	$04, $01, $00	;  500
		.byte	$05, $01, $00	;  700
		.byte	$08, $07, $06	; 1000
		.byte	$09, $07, $06	; 2000
		.byte	$0A, $07, $06	; 3000
		.byte	$0B, $07, $06	; 5000
		;
MAZESAVE:	.res $15 * $16

;*********************************************************************
; VIC Custom Graphic Characters
;
; If < 64 will be used for the software sprite stack, the remaining
; unused characters can be used for other custom graphics, beginning
; at $1C00 where the "@" symbol can be redefined.
;
; Do not use this as an initialized segment if you plan on linking
; this source as a future game cartridge later.  You must COPY any
; read-only data into this address space.
;
; If your data was saved from some tool in binary format, you can
; include that binary file here:
;		.incbin "graphics.dat"
;
; else, just enter each 8x8 values here, such as:
;		.byte	$FF,$81,$81,$81,$81,$81,$81,$FF
; or:
;		.byte	%11111111	; square
;		.byte	%10000001
;		.byte	%10000001
;		.byte	%10000001
;		.byte	%10000001
;		.byte	%10000001
;		.byte	%10000001
;		.byte	%11111111
;
		.segment "MYCHAR"

FS00:	.byte	%00000000
		.byte	%10000000
		.byte	%01000000
		.byte	%01000000
		.byte	%01000000
		.byte	%01000000
		.byte	%10000000
		.byte	%00000000
		;
		.byte	%00000000
		.byte	%00110001
		.byte	%01001010
		.byte	%01001010
		.byte	%01001010
		.byte	%01001010
		.byte	%00110001
		.byte	%00000000
		;
FS100:	.byte	%00000000
		.byte	%00000010
		.byte	%00000110
		.byte	%00000010
		.byte	%00000010
		.byte	%00000010
		.byte	%00000111
		.byte	%00000000
		;
FS300:	.byte	%00000000
		.byte	%00001111
		.byte	%00000001
		.byte	%00000110
		.byte	%00000001
		.byte	%00001001
		.byte	%00000110
		.byte	%00000000
		;
FS500:	.byte	%00000000
		.byte	%00001111
		.byte	%00001000
		.byte	%00001110
		.byte	%00000001
		.byte	%00001001
		.byte	%00000110
		.byte	%00000000
		;
FS700:	.byte	%00000000
		.byte	%00001111
		.byte	%00000001
		.byte	%00000010
		.byte	%00000100
		.byte	%00000100
		.byte	%00000100
		.byte	%00000000
		;
FS000:	.byte	%00000000
		.byte	%00011000
		.byte	%10100100
		.byte	%10100100
		.byte	%10100100
		.byte	%10100100
		.byte	%00011000
		.byte	%00000000
		;
		.byte	%00000000
		.byte	%01100011
		.byte	%10010100
		.byte	%10010100
		.byte	%10010100
		.byte	%10010100
		.byte	%01100011
		.byte	%00000000
		;
FS1000:	.byte	%00000000
		.byte	%00000100
		.byte	%00001100
		.byte	%00000100
		.byte	%00000100
		.byte	%00000100
		.byte	%00001110
		.byte	%00000000
		;
FS2000:	.byte	%00000000
		.byte	%00001100
		.byte	%00010010
		.byte	%00000010
		.byte	%00001100
		.byte	%00010000
		.byte	%00011110
		.byte	%00000000
		;
FS3000:	.byte	%00000000
		.byte	%00011110
		.byte	%00000010
		.byte	%00001100
		.byte	%00000010
		.byte	%00010010
		.byte	%00001100
		.byte	%00000000
		;
FS5000:	.byte	%00000000
		.byte	%00011110
		.byte	%00010000
		.byte	%00011100
		.byte	%00000010
		.byte	%00010010
		.byte	%00001100
		.byte	%00000000
		;
GHOST0:	.byte	$00, $38, $7C, $54, $7C, $7C, $54, $00	; $60 small
GHOST1:	.byte	$1C, $3E, $7F, $64, $7F, $7F, $7F, $55	; $68 right
GHOST2:	.byte	$1C, $3E, $7F, $7F, $49, $7F, $7F, $55	; $70 down
GHOST3:	.byte	$38, $7C, $FE, $26, $FE, $FE, $FE, $AA	; $78 left
GHOST4:	.byte	$38, $7C, $FE, $FE, $FE, $FE, $FE, $AA	; $80 up
GHOST5:	.byte	$38, $7C, $FE, $92, $FE, $82, $FE, $AA	; $88 caged
GHOST6:	.byte	$38, $7C, $FE, $92, $FE, $82, $FE, $54	; $90 fleeing
QMAN:	.byte	$3C, $7E, $BD, $FF, $BD, $C3, $7E, $3C	; $98 smiley
QMAN0:	.byte	$3C, $7E, $FF, $FF, $FF, $FF, $7E, $3C	; $A0 closed
QMAN1:	.byte	$3E, $7C, $F8, $F0, $F0, $F8, $7C, $3E	; $A8 right
QMAN2:	.byte	$3C, $7E, $FF, $FF, $E7, $C3, $81, $00	; $B0 down
QMAN3:	.byte	$7C, $3E, $1F, $0F, $0F, $1F, $3E, $7C	; $B8 left
QMAN4:	.byte	$00, $81, $C3, $E7, $FF, $FF, $7E, $3C	; $C0 up
POOF1:	.byte	$00, $10, $10, $6C, $10, $10, $00, $00	; $C8 explosion
POOF2:	.byte	$10, $44, $28, $C6, $28, $44, $10, $00	; $D0 smoke
POOF3:	.byte	$92, $44, $00, $82, $00, $44, $92, $00	; $D8 dust
		; original
		.byte	%00000111
		.byte	%00000011
		.byte	%00110101
		.byte	%01001000
		.byte	%10000100
		.byte	%10000100
		.byte	%01001000
		.byte	%00110000
		; alternative
		.byte	%00111100
		.byte	%01000010
		.byte	%10000001
		.byte	%01000010
		.byte	%00111100
		.byte	%00011000
		.byte	%00111100
		.byte	%00011000
		.byte	$00, $00, $00, $18, $18, $00, $00, $00	; $F0 dot
		.byte	$00, $3C, $7E, $7E, $7E, $7E, $3C, $00	; $F8 powerpill (animated)
GRAPHICS3:
		.byte	$00, $00, $00, $00, $00, $00, $00, $00	; $00 empty space
		.byte	$04, $08, $18, $24, $62, $F7, $F2, $60	; $08 cherry
		.byte	$10, $7C, $FE, $AA, $D6, $AA, $54, $28	; $10 strawberry
		.byte	$20, $10, $7C, $FE, $FE, $FE, $7C, $38	; $18 peach
		.byte	$08, $10, $7C, $FE, $FE, $FE, $7C, $28	; $20 apple
		.byte	$08, $10, $38, $38, $7C, $FE, $FE, $6C	; $28 pear
		.byte	$10, $30, $92, $FE, $7C, $38, $10, $28	; $30 tbird
		.byte	$10, $38, $7C, $7C, $7C, $7C, $FE, $10	; $38 bell
		.byte	$18, $24, $18, $08, $08, $18, $08, $18	; $40 key
		.byte	$00, $FF, $01, $01, $01, $01, $FE, $00	; $48 doorway east
		.byte	$00, $FF, $80, $80, $80, $80, $7F, $00	; $50 doorway west
		.byte	$00, $FF, $00, $00, $00, $00, $00, $00	; $58 maze wall h-top
		.byte	$00, $00, $00, $00, $00, $00, $FF, $00	; $60 maze wall h-bottom
		.byte	$40, $40, $40, $40, $40, $20, $1F, $00	; $68 maze wall s-w corner
		.byte	$02, $02, $02, $02, $02, $04, $F8, $00	; $70 maze wall s-e corner
		.byte	$00, $1F, $20, $40, $40, $40, $40, $40	; $78 maze wall n-w corner
		.byte	$00, $F8, $04, $02, $02, $02, $02, $02	; $80 maze wall n-e corner
		.byte	$00, $18, $24, $42, $42, $42, $42, $42	; $88 maze wall north
		.byte	$42, $42, $42, $42, $42, $24, $18, $00	; $90 maze wall south
		.byte	$00, $1F, $20, $40, $40, $20, $1F, $00	; $98 maze wall west
		.byte	$00, $F8, $04, $02, $02, $04, $F8, $00	; $A0 maze wall east
		.byte	$42, $41, $40, $40, $40, $20, $1F, $00	; $A8 maze wall s-w elbow
		.byte	$42, $82, $02, $02, $02, $04, $F8, $00	; $B0 maze wall s-e elbow
		.byte	$00, $1F, $20, $40, $40, $40, $41, $42	; $B8 maze wall n-w elbow
		.byte	$00, $F8, $04, $02, $02, $02, $82, $42	; $C0 maze wall n-e elbow
		.byte	$42, $42, $42, $42, $42, $42, $42, $42	; $C8 maze wall vertical
		.byte	$00, $FF, $00, $00, $00, $00, $FF, $00	; $D0 maze wall horizontal
		.byte	$42, $41, $40, $40, $40, $40, $41, $42	; $D8 maze wall west tee
		.byte	$42, $82, $02, $02, $02, $02, $82, $42	; $E0 maze wall east tee
		.byte	$00, $FF, $00, $00, $00, $00, $81, $42	; $E8 maze wall north tee
		.byte	$42, $81, $00, $00, $00, $00, $FF, $00	; $F0 maze wall south tee
		.byte	$42, $81, $00, $00, $00, $00, $81, $42	; $F8 maze wall cross
GRAPHICS4:
FKEYL:	.byte	%00001111
		.byte	%00001111
		.byte	%00001111
		.byte	%00001111
		.byte	%00001111
		.byte	%00001111
		.byte	%00001111
		.byte	%00001111
FKEY:	.byte	%11111011
		.byte	%11101111
		.byte	%11101111
		.byte	%10101011
		.byte	%11101111
		.byte	%11101111
		.byte	%11101111
		.byte	%11101111
FKEYB:	.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		.byte	%00000000
		.byte	%00000000
		.byte	%00000000
		.byte	%00000000
F1:		.byte	%11111011
		.byte	%11101011
		.byte	%11111011
		.byte	%11111011
		.byte	%11111011
		.byte	%11111011
		.byte	%11111011
		.byte	%11111011
F3:		.byte	%11101011
		.byte	%10111110
		.byte	%11111110
		.byte	%11101011
		.byte	%11111110
		.byte	%11111110
		.byte	%10111110
		.byte	%11101011
F5:		.byte	%10101010
		.byte	%10111111
		.byte	%10111111
		.byte	%10101011
		.byte	%11111110
		.byte	%11111110
		.byte	%10111110
		.byte	%11101011
F7:		.byte	%10101010
		.byte	%11111110
		.byte	%11111110
		.byte	%11111011
		.byte	%11111011
		.byte	%11101111
		.byte	%11101111
		.byte	%11101111
		; regulator table (bit=1 means wait for next vsync)
SPEED:	.byte	%11101110	; NTSC:cherry, PAL:n/a
		.byte	%11010110	; NTSC:strawberry, PAL:cherry
		.byte	%01010101	; NTSC:peach #1, PAL:strawberry
		.byte	%00101001	; NTSC:peach #2, PAL:peach #1
		.byte	%00010001	; NTSC:apple #1, PAL:peach #2
		.byte	%00010000	; NTSC:n/a, PAL:apple #1
		.byte	%00000000
		.byte	%00000000
		; CHERRY
		.byte	%00000000
		.byte	%00000000
		.byte	%00000000
		.byte	%00000000
		.byte	%00000011
		.byte	%00000011
		.byte	%00001100
		.byte	%00001100
		;
		.byte	%00101010
		.byte	%10101010
		.byte	%10101010
		.byte	%10101010
		.byte	%10101010
		.byte	%10101010
		.byte	%10101010
		.byte	%00101010
		;
		.byte	%00110000
		.byte	%00110000
		.byte	%11110000
		.byte	%11110000
		.byte	%00110000
		.byte	%00110000
		.byte	%00110000
		.byte	%00110000
		;
		.byte	%00101000
		.byte	%10101010
		.byte	%10101010
		.byte	%10101010
		.byte	%10101010
		.byte	%10101010
		.byte	%00101000
		.byte	%00000000
		; BIG DUDE
		.byte	%00000101
		.byte	%00010101
		.byte	%01010101
		.byte	%01010101
		.byte	%01101001
		.byte	%01101001
		.byte	%01111001
		.byte	%01111001
		;
		.byte	%01010101
		.byte	%01010101
		.byte	%01010001
		.byte	%01000100
		.byte	%01010101
		.byte	%01010101
		.byte	%01000100
		.byte	%01000100
		;
		.byte	%01010000
		.byte	%01010100
		.byte	%01010101
		.byte	%01010101
		.byte	%01101001
		.byte	%01101001
		.byte	%01111001
		.byte	%01111001
		;
		.byte	%01010101
		.byte	%01010101
		.byte	%01000101
		.byte	%00010001
		.byte	%01010101
		.byte	%01010101
		.byte	%00010001
		.byte	%00010001
		; QUIKMAN
		.byte	%00000111
		.byte	%00011111
		.byte	%00111111
		.byte	%01111111
		.byte	%01111111
		.byte	%11111111
		.byte	%11111111
		.byte	%11111110
		;
		.byte	%11111110
		.byte	%11111111
		.byte	%11111111
		.byte	%01111111
		.byte	%01111111
		.byte	%00111111
		.byte	%00011111
		.byte	%00000111
		;
		.byte	%11100000
		.byte	%11111000
		.byte	%11110000
		.byte	%11100000
		.byte	%11000000
		.byte	%10000000
		.byte	%00000000
		.byte	%00000000
		;
		.byte	%00000000
		.byte	%00000000
		.byte	%10000000
		.byte	%11000000
		.byte	%11100000
		.byte	%11110000
		.byte	%11111000
		.byte	%11100000
		; QUICKMAN2
		.byte	%00000111
		.byte	%00011111
		.byte	%00111111
		.byte	%01111111
		.byte	%01111111
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		;
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		.byte	%01111111
		.byte	%01111111
		.byte	%00111111
		.byte	%00011111
		.byte	%00000111
		;
		.byte	%11100000
		.byte	%11111000
		.byte	%11111100
		.byte	%11111110
		.byte	%11111100
		.byte	%11110000
		.byte	%11000000
		.byte	%00000000
		;
		.byte	%00000000
		.byte	%11000000
		.byte	%11110000
		.byte	%11111100
		.byte	%11111110
		.byte	%11111100
		.byte	%11111000
		.byte	%11100000
		; QUICKMAN3
		.byte	%00000111
		.byte	%00011111
		.byte	%00111111
		.byte	%01111111
		.byte	%01111111
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		;
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		.byte	%01111111
		.byte	%01111111
		.byte	%00111111
		.byte	%00011111
		.byte	%00000111
		;
		.byte	%11100000
		.byte	%11111000
		.byte	%11111100
		.byte	%11111110
		.byte	%11111110
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		;
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		.byte	%11111110
		.byte	%11111110
		.byte	%11111100
		.byte	%11111000
		.byte	%11100000
		; QUICKMAN4
		.byte	%00000111
		.byte	%00011111
		.byte	%00111111
		.byte	%01111111
		.byte	%01111111
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		;
		.byte	%11111111
		.byte	%11111111
		.byte	%11111111
		.byte	%01111111
		.byte	%01111111
		.byte	%00111111
		.byte	%00011111
		.byte	%00000111
		;
		.byte	%11100000
		.byte	%11111000
		.byte	%11111100
		.byte	%11111110
		.byte	%11111100
		.byte	%11110000
		.byte	%11000000
		.byte	%00000000
		;
		.byte	%00000000
		.byte	%11000000
		.byte	%11110000
		.byte	%11111100
		.byte	%11111110
		.byte	%11111100
		.byte	%11111000
		.byte	%11100000
		; copyright
		.byte	%00111100
		.byte	%01000010
		.byte	%10011101
		.byte	%10100001
		.byte	%10100001
		.byte	%10011101
		.byte	%01000010
		.byte	%00111100

;*********************************************************************
; Your program code starts here
;
		.segment "CODE"

READY:
		JMP RESET	; enter VIC BASIC mode

RESTART:
		.include "quikman.s"
